﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Account.Form', {
    extend: 'System.Control.EntityForm',
    entityType: 'Kivii.Finances.Entities.Account',
    xtype: 'App.Control.Kivii.Finances.Account.Form',
    autoScroll: true,
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 800,
            large: 0
        }
    },
    viewModel: {
        data: {
            entity: {
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                Currency: 'CNY'
                // OwnerName: KiviiContext.CurrentMember.OrganizationName,
                // OwnerKvid: KiviiContext.CurrentMember.OrganizationKvid
            }
        },
        formulas: {
            amountChineseBig: function(get) {
                var _amount = get('entity.Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            }
        }
    },
    tbar: ['->', {
        text: '保存',
        iconCls: 'x-fa fa-save',
        iconAlign: 'left',
        handler: '_onSaveEntity',
        bind: {
            disabled: '{!formIsValid}'
        }
    }],
    defaults: {
        // margin: '0 10 10 10',
        labelWidth: 60,
        labelAlign: 'right',
        xtype: 'textfield'
    },
    items: [{
        xtype: 'comboSelector',
        fieldLabel: '所属部门',
        responsiveCls: 'big-100 small-100',
        dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
        editable: false,
        allowBlank: false,
        readOnly: true,
        keyField: 'Kvid',
        valueField: 'FullName',
        bind: {
            key: '{entity.OwnerKvid}',
            value: '{entity.OwnerName}'
        }
        // },{
        //     fieldLabel: '所属部门',
        //     bind: '{entity.OwnerName}',
        //     responsiveCls: 'big-50 small-100',
        //     readOnly: true
    }, {
        fieldLabel: '开户名称',
        bind: '{entity.Name}',
        responsiveCls: 'big-100 small-100',
        reference:'textName',
        hidden:true,
        allowBlank: false
    },{
        xtype: 'windowSelector',
        fieldLabel: '开户名称',
        responsiveCls: 'big-100 small-100',
        hidden:false,
        reference:'selectName',
        editable: false,
        keyField: 'Kvid',
        valueField: 'FullName',
        value1Field: 'FullName',
        window: {
            type: 'App.Control.Kivii.Finances.Settlement.Customer.Selector'
        },
        allowBlank: false,
        bind: {
            key: '{entity.BizKvid}',
            value: '{entity.Name}',
            value1:'{entity.BizId}'
        },
        listeners: {
            entitySelected: '_onSelectName'
        }
    }, {
        fieldLabel: '开户账号',
        responsiveCls: 'big-100 small-100',
        allowBlank: false,
        bind: {
            value: '{entity.SerialNumber}'
        }
    }, {
        xtype: 'combo',
        fieldLabel: '账户类型',
        editable: false,
        allowBlank: false,
        responsiveCls: 'big-50 small-100',
        // readOnly: true,
        displayField: 'dis',
        valueField: 'val',
        store: {
            fields: ['val', 'dis'],
            data: [{
                //     val: 'Unsupported',
                //     dis: '不支持'
                // }, {
                //     val: 'Cash',
                //     dis: '现金账户'
                // }, {
                //     val: 'Pos',
                //     dis: '刷卡账户'
                // }, {
                //     val: 'AliPay',
                //     dis: '支付宝账户'
                // }, {
                //     val: 'WeChat',
                //     dis: '微信账户'
                // }, {
                //     val: 'Bank',
                //     dis: '银行账户'
                // }, {
                val: 'Deposit',
                dis: '存款账户'
                // }, {
                //     val: 'Coupon',
                //     dis: '优惠券'
                // }, {
                //     val: 'StoredCard',
                //     dis: '储值卡'
                // }, {
                //     val: 'Biz',
                //     dis: '业务专用账户'
                // }, {
                //     val: 'Discount',
                //     dis: '折扣账户'
                // }, {
                //     val: 'Balance',
                //     dis: 'Balance'
            }]
        },
        bind: {
            value: '{entity.Type}'
        }
    }, {
        xtype: 'combo',
        fieldLabel: '币种单位',
        editable: false,
        allowBlank: false,
        responsiveCls: 'big-50 small-100',
        // readOnly: true,
        displayField: 'dis',
        valueField: 'val',
        store: {
            fields: ['val', 'dis'],
            data: [{
                val: 'CNY',
                dis: '人民币'
            }, {
                val: 'USD',
                dis: '美元'
            }]
        },
        bind: {
            value: '{entity.Currency}'
        }
    }],
    controller: {
        _onSelectName: function(_ctrl, _entity) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _serialNumber = _entity[0].Kvid.substring(0,8) + new Date().getTime();
            _vm.set('entity.SerialNumber', _serialNumber);
            _vm.set('entity.BizType','Kivii.Customers.Entities.Customer');
        },
        _onSaveEntity: function(_ctrl) {
            var _me = this,
                _view = _me.getView(),
                _vm = _view.getViewModel();
            _view.saveEntity();
        }
    }
});