﻿//解决rowwidget不能触发collapsebody和expandbody事件
Ext.override(Ext.grid.plugin.RowExpander, {
    relayedEvents: ['collapsebody', 'expandbody'],
    init: function(grid) {
        //--给grid注册expandbody 和collapsebody事件
        var me = this,
            ownerLockable = grid.ownerLockable;
        grid.rowExpanderEventRelayers = grid.relayEvents(grid.view, me.relayedEvents);
        if (ownerLockable) {
            ownerLockable.editorEventRelayers = ownerLockable.relayEvents(ownerLockable.view, me.relayedEvents);
        }
        this.callParent([grid]);

    },
    //重写getHeaderConfig，使下级没有数据时，可以隐藏加号,通过haveRowExpander判断
    getHeaderConfig: function() {
        var me = this,
            lockable = me.grid.lockable && me.grid;

        return {
            width: me.headerWidth,
            ignoreExport: true,
            lockable: false,
            autoLock: true,
            sortable: false,
            resizable: false,
            draggable: false,
            hideable: false,
            menuDisabled: true,
            tdCls: Ext.baseCSSPrefix + 'grid-cell-special',
            innerCls: Ext.baseCSSPrefix + 'grid-cell-inner-row-expander',
            renderer: function(value, meta, record) {
                if (me.haveRowExpander) {
                    if (me.haveRowExpander(record)) {
                        return '<div class="' + Ext.baseCSSPrefix + 'grid-row-expander" role="presentation" tabIndex="0"></div>';
                    } else {
                        return "";
                    }
                } else {
                    return '<div class="' + Ext.baseCSSPrefix + 'grid-row-expander" role="presentation" tabIndex="0"></div>';
                }
            },
            processEvent: function(type, view, cell, rowIndex, cellIndex, e, record) {
                var isTouch = e.pointerType === 'touch',
                    isExpanderClick = !!e.getTarget('.' + Ext.baseCSSPrefix + 'grid-row-expander');

                if ((type === "click" && isExpanderClick) || (type === 'keydown' && e.getKey() === e.SPACE)) {

                    // Focus the cell on real touch tap.
                    // This is because the toggleRow saves and restores focus
                    // which may be elsewhere than clicked on causing a scroll jump.
                    if (isTouch) {
                        cell.focus();
                    }
                    me.toggleRow(rowIndex, record, e);
                    e.stopSelection = !me.selectRowOnExpand;
                } else if (e.type === 'mousedown' && !isTouch && isExpanderClick) {
                    e.preventDefault();
                }
            },

            // This column always migrates to the locked side if the locked side is visible.
            // It has to report this correctly so that editors can position things correctly
            isLocked: function() {
                return lockable && (lockable.lockedGrid.isVisible() || this.locked);
            },

            // In an editor, this shows nothing.
            editRenderer: function() {
                return ' ';
            }
        };
    }

});
Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Account.Pay.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Pay',
    xtype: 'App.Control.Kivii.Finances.Account.Pay.List',
    title: '业务记录',
    iconCls: 'x-fa fa-ticket',
    globalFilter: {
        OffsetKvid: '00000000-0000-0000-0000-000000000000'
    },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar: true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer: false,
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'CreateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                xtype: 'datefield',
                fieldLabel: '操作日期 开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanBeginTime'
                    }
                },
                // value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanEndTime'
                    }
                },
                // value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'BizId,PayerName,PayeeName',
                // queryField: 'Amount',
                emptyText: '模糊搜索业务号、收付款单位称',
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '对方单位',
            dataIndex: 'PayerName',
            renderer:'_renderPayer',
            width: 240
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'AmountPayment',
            renderer:'_renderAmountPayment',
            width: 110,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
        	text: '付款来源',
            dataIndex: 'FromType',
            renderer:'_renderFromType',
            width: 100
    	},{
            text: '抹零金额',
            align: 'right',
            dataIndex: 'AmountDiscount',
            width: 110,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            text: '到账日期',
            dataIndex: 'PayedTime',
            width: 100,
            renderer: function(_value) {
                return Ext.util.Format.date(_value, 'Y-m-d');
            }
        }, {
            text: '操作日期',
            dataIndex: 'OperateTime',
            width: 100,
            renderer: function(_value) {
                return Ext.util.Format.date(_value, 'Y-m-d');
            }
        }, {
        	text: '分类',
            dataIndex: 'Type',
            renderer:'_renderType',
            width: 100
    	}, {
            text: '结算业务',
            dataIndex: 'BizId',
            width: 150
        }, {
        	text: '摘要',
            dataIndex: 'Summary',
            width: 150
    	},{
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 200,
            flex: 1
        }]
    },
    plugins: [{
        ptype: 'gridfilters'
    }, {
        ptype: 'rowwidget',
        widget: {
            xtype: 'grid',
            reference: 'payment',
            autoLoad: true,
            store: {
                pageSize: 10000,
                data: [],
                autoLoad: false
            },
            border: '1px #d0d0d0 solid',
            margin: '0 0 0 20',
            features: [{
                ftype: 'summary'
            }],
            columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '流水号',
            dataIndex: 'SerialNumber',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            width: 220
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: '_renderAmount',
            width: 110
        }, {
            text: '类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        }, {
            xtype: 'datecolumn',
            text: '操作日期',
            format: 'Y-m-d H:i',
            dataIndex: 'CreateTime',
            width: 150
        }, {
            text: '操作人',
            dataIndex: 'OperatorName',
            width: 100
        }, {
            text: '充值账户',
            dataIndex: 'PayeeAccountName',
            width: 280
        }, {
            text: '账户所属',
            dataIndex: 'PayeeAccountOwnerName',
            width: 150
        }, {
            text: '充值账号',
            dataIndex: 'PayeeAccountSerialNumber',
            width: 180
        }, {
            text: '备注',
            dataIndex: 'Remark',
            flex: 1
        }]
    },
        },
        //配置该方法,返回true时,可以展开,否则不能展开
        haveRowExpander:function(_record){
        	if(_record.data.Type==="Transfer")return true;
        	return false;
        }
    }],
    
    listeners: {
        activate: '_onActivate',
        expandbody: '_onExpandBody'
    },
    setOwner: function(_owner) {
        let _view = this;
        let _vm = _view.getViewModel();
        _vm.set('_owner', _owner);
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderObject: function(_value, _meta, _record, _rowIndex, _cellIndex, _store, _grid) { //对Metadata的Render的默认方法
            var _dataIndex = _meta.column.dataIndex;
            if (!_dataIndex) return _value;
            var _pos = _dataIndex.indexOf('.');
            if (_pos < 1) return _value;
            var _objName = _dataIndex.substr(0, _pos);
            if (!_record.data[_objName]) return '';
            if (!Ext.isObject(_record.data[_objName])) return '';
            var _propertyName = _dataIndex.substr(_pos + 1);
            return _record.data[_objName][_propertyName];
        },
        _renderPayer:function(_value, _meta, _record, _rowIndex) {
        	let _controller=this;
        	let _vm=_controller.getViewModel();
        	let _owner=_vm.data._owner;
            if (_record.data.PayerKvid === _owner.Kvid) {
            	_meta.style = 'background-color:#F0E68C';
                return _record.data.PayeeName;
            }
            return _value;
        },
        _renderAmountPayment:function(_value, _meta, _record, _rowIndex) {
        	let _controller=this;
        	let _vm=_controller.getViewModel();
        	let _owner=_vm.data._owner;
            if (_record.data.PayerKvid === _owner.Kvid) {
            	_meta.style = 'background-color:#F0E68C';
                return 0-_value;
            }
            return _value;
        },
        _renderFromType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Account") {
                return "账户";
            }
            if (_value === "Payment") {
                return "到账流水";
            }
            return _value;
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Transfer") {
                return "充值";
            }
            if (_value === "DynamicPay") {
                return "收款登记";
            }
            return _value;
        },
        _renderAmountDetail: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            } else _meta.style = 'background-color:#F0E68C';
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== null) _meta.style = 'background-color:#FF0000';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== null) _meta.style = 'background-color:#FF0000';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        //点击+号,展示详情
        _onExpandBody: function(_rowNode, _record, _expandRow, _eOpts) {
            let _controller = this;
            let _refs = _controller.getReferences();
            let _view = _controller.getView();
            let _storeSettlement = _refs.payment.getStore();
            _storeSettlement.removeAll();
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Payment/UsedQuery.json',
                params: {
                    Kvid: _record.data.FromKvid
                },
                success: function(_response) {
                    if (_response.responseText && _response.responseText.length > 0) {
                        let _rtns = Ext.decode(_response.responseText);
                        for (let _i = 0; _i < _rtns.Results.length; _i++) {
                            _storeSettlement.add(_rtns.Results[_i]);
                        }
                    }
                },
                failure: function(_response, _opts) {}
            });
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            delete _view.filter.OperateTimeLessThan;
            _view.filter.OperateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.OperateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _cleanBeginTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            _view.reload();
        },
        _cleanEndTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeLessThan;
            _view.reload();
        }
    }
});