﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Invoice.Form', {
    extend: 'System.Control.EntityForm',
    xtype: 'App.Control.Kivii.Finances.Invoice.Form',
    entityType: 'Kivii.Finances.Entities.Invoice',
    // requires: ['Package.ux'],
    layout: {
        type: 'vbox',
        pack: 'start',
        align: 'stretch'
    },
    autoScroll: true,
    id: 'execute',
    viewModel: {
        data: {
            isEdit: false,
            entity: {
                Metadata: {},
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                Kvid: null
            },
            payments: []
        },
        formulas: {
            hiddenAccept: function(get) {
                let _kvid = get('entity.Kvid');
                let _isEdit = get('isEdit');
                if (_isEdit) return _isEdit;
                if (_kvid === null) return false;
                return true;
            },
            hiddenSave: function(get) {
                let _kvid = get('entity.Kvid');
                let _isEdit = get('isEdit');
                if (_isEdit) return _isEdit;
                if (_kvid !== null) return false;
                return true;
            },
            hiddenEdit: function(get) {
                let _isEdit = get('isEdit');
                let _kvid = get('entity.Kvid');
                if (_kvid !== null) return true;
                return !_isEdit;
            },
            amountChineseBig: function(get) {
                let _amount = get('entity.Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            }
        }
    },
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        items: [{
            margin: '0 0 -9 3',
            xtype: 'displayfield',
            bind: {
                value: '录入人:{owner.CreatorName},总金额:{owner.Amount}'
            },
            fieldStyle: {
                fontSize: '20px'
            }

        }, '->', {
            text: '保存',
            iconCls: 'x-fa fa-save',
            iconAlign: 'left',
            handler: '_onSaveEntity',
            bind: {
                disabled: '{!formIsValid}',
                hidden: '{hiddenSave}'
            }
        }, {
            text: '开票',
            iconCls: 'x-fa fa-save',
            iconAlign: 'left',
            handler: '_onAcceptEntity',
            bind: {
                disabled: '{!formIsValid}',
                hidden: '{hiddenAccept}'
            }
        }, {
            text: '编辑',
            iconCls: 'x-fa fa-edit',
            iconAlign: 'left',
            handler: '_onEditEntity',
            bind: {
                disabled: '{!formIsValid}',
                hidden: '{hiddenEdit}'
            }
        }]
    }],
    items: [{
        xtype: 'container',
        // flex:1,
        // height: 140,
        margin: '10 0 0 0',
        defaults: {
            xtype: 'textfield',
            columnWidth: 1 / 4,
            labelWidth: 65,
            labelAlign: 'right',
            // bind: {
            //     readOnly: '{entity.Kvid}'
            // },
            margin: '0 10 10 10'
        },
        layout: 'column',
        items: [{
            fieldLabel: '发票代码',
            allowBlank: false,
            columnWidth: 1 / 3,
            bind: {
                value: '{entity.SerialCode}'
            }
        }, {
            fieldLabel: '发票号码',
            allowBlank: false,
            columnWidth: 1 / 3,
            bind: {
                value: '{entity.SerialNumber}'
            }
        }, {
            xtype: 'comboSelector',
            fieldLabel: '所属部门',
            columnWidth: 1 / 3,
            editable: false,
            allowBlank: false,
            dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
            reference: 'depart',
            keyField: 'Kvid',
            valueField: 'FullName',
            key: KiviiContext.CurrentMember.DepartmentKvid,
            value: KiviiContext.CurrentMember.DepartmentName,
            bind: {
                key: '{entity.OwnerKvid}',
                value: '{entity.OwnerName}'
            }
        }, {
            xtype: 'categoryField',
            fieldLabel: '发票税率',
            emptyText: '请选择数字',
            // hideTrigger: true,
            columnWidth: 1 / 3,
            allowBlank: false,
            bind: {
            	// readOnly: '{entity.Kvid}',
            	value:'{entity.TaxRate}'
            },
            internalCode: 'TaxRate',
            entityType: 'Kivii.Finances.Entities.InvoiceApplyDetail'
            // },{
            // xtype: 'numberfield',
            // fieldLabel: '发票税率',
            // emptyText: '请输入数字',
            // hideTrigger: true,
            // columnWidth: 1 / 3,
            // minValue: 0,
            // maxValue: 1,
            // step: 0.1,
            // decimalPrecision: 2,
            // // readOnly:false,
            // bind: {
            //     value: '{entity.TaxRate}'
            // },
            // allowBlank: false
        }, {
            xtype: 'categoryField',
            fieldLabel: '发票类型',
            allowBlank: false,
            columnWidth: 1 / 3,
            internalCode: 'Type',
            bind: {
                value: '{entity.Type}'
            }
        }, {
            xtype: 'datefield',
            format: 'Y-m-d',
            altFormats: 'D, d M Y H:i:s T|c|Y-m-d|Y-m-d H:i|Y/n/j',
            fieldLabel: '开票日期',
            editable: false,
            columnWidth: 1 / 3,
            allowBlank: false,
            bind: {
                value: '{entity.OperateTime}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            fieldLabel: '收&nbsp;&nbsp;款&nbsp;人',
            allowBlank: false,
            columnWidth: 1 / 3,
            bind: {
                value: '{entity.Metadata.PayeeOperatorName}'
            }
        }, {
            fieldLabel: '复&nbsp;&nbsp;核&nbsp;人',
            allowBlank: false,
            columnWidth: 1 / 3,
            bind: {
                value: '{entity.Metadata.ReviewerName}'
            }
        }, {
            fieldLabel: '开&nbsp;&nbsp;票&nbsp;人',
            allowBlank: false,
            columnWidth: 1 / 3,
            bind: {
                value: '{entity.OperatorName}'
            }
        }, {
            fieldLabel: '摘&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;要',
            columnWidth: 1,
            bind: {
                value: '{entity.Summary}'
            }
        }, {
            fieldLabel: '备&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;注',
            emptyText: '备注信息会在开具的发票备注栏体现',
            columnWidth: 1,
            bind: {
                value: '{entity.Remark}'
            }
        }]
    }, {
        xtype: 'container',
        height: 200,
        layout: 'column',
        items: [{
            xtype: 'container',
            margin: '10 5 10 10',
            height: 180,
            columnWidth: 1 / 2,
            style: {
                border: '1px solid #C1C1C1'
            },
            layout: 'hbox',
            items: [{
                xtype: 'container',
                height: 180,
                width: 35,
                style: {
                    'border-right': '0.5px solid #C1C1C1'
                },
                html: '<div style="font-size:16px;margin-top:38px;margin-left:10px;width:10px;word-wrap:break-word;">&nbsp;&nbsp;购&nbsp;&nbsp;买&nbsp;&nbsp;方&nbsp;&nbsp;</div>'
            }, {
                xtype: 'container',
                height: 180,
                flex: 1,
                layout: 'column',
                defaults: {
                    xtype: 'textfield',
                    columnWidth: 1 / 2,
                    labelWidth: 65,
                    labelAlign: 'right',
                    margin: '0 10 10 10',
                    readOnly: true
                },
                items: [{
                    xtype: 'windowSelector',
                    columnWidth: 1,
                    margin: '10 10 5 10',
                    fieldLabel: '名&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;称',
                    editable: false,
                    readOnly: false,
                    allowBlank: false,
                    keyField: 'Kvid',
                    valueField: 'CompanyName',
                    value1Field: 'TaxNumber',
                    value2Field: 'CompanyAddress',
                    value3Field: 'Phone',
                    value4Field: 'BankName',
                    value5Field: 'BankAccount',
                    window: {
                        type: 'App.Control.Kivii.Finances.InvoiceTitle.Selector'
                    },
                    Type: 'Payee',
                    bind: {
                        key: '{entity.PayerKvid}',
                        value: '{entity.PayerName}',
                        value1: '{entity.PayerTaxNumber}',
                        value2: '{entity.PayerCompanyAddress}',
                        value3: '{entity.PayerPhone}',
                        value4: '{entity.PayerRegisteredBank}',
                        value5: '{entity.PayerBankAccount}'
                        // readOnly: '{entity.Kvid}'
                    }
                }, {
                    fieldLabel: '纳&nbsp;&nbsp;税&nbsp;号',
                    margin: '0 10 5 10',
                    allowBlank: false,
                    columnWidth: 1,
                    bind: '{entity.PayerTaxNumber}'
                }, {
                    fieldLabel: '地&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;址',
                    bind: '{entity.PayerCompanyAddress}'
                }, {
                    fieldLabel: '电&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;话',
                    bind: '{entity.PayerPhone}'
                }, {
                    fieldLabel: '开&nbsp;&nbsp;户&nbsp;行',
                    bind: '{entity.PayerRegisteredBank}'
                }, {
                    fieldLabel: '账&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;号',
                    bind: '{entity.PayerBankAccount}'
                }]
            }]
        }, {
            xtype: 'container',
            margin: '10 10 10 5',
            height: 180,
            columnWidth: 1 / 2,
            style: {
                border: '1px solid #C1C1C1'
            },
            layout: 'hbox',
            items: [{
                xtype: 'container',
                height: 180,
                width: 35,
                style: {
                    'border-right': '0.5px solid #C1C1C1'
                },
                html: '<div style="font-size:16px;margin-top:38px;margin-left:10px;width:10px;word-wrap:break-word;">&nbsp;&nbsp;销&nbsp;&nbsp;售&nbsp;&nbsp;方&nbsp;&nbsp;</div>'
            }, {
                xtype: 'container',
                height: 180,
                flex: 1,
                layout: 'column',
                defaults: {
                    xtype: 'textfield',
                    columnWidth: 1 / 2,
                    labelWidth: 65,
                    labelAlign: 'right',
                    margin: '0 10 10 10',
                    readOnly: true
                },
                items: [{
                    xtype: 'comboSelector',
                    columnWidth: 1,
                    margin: '10 10 5 10',
                    fieldLabel: '名&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;称',
                    dataSource: '/codet/CurrentMember/InvoiceTitle.json',
                    editable: false,
                    readOnly: false,
                    allowBlank: false,
                    keyField: 'Kvid',
                    valueField: 'CompanyName',
                    value1Field: 'TaxNumber',
                    value2Field: 'CompanyAddress',
                    value3Field: 'Phone',
                    value4Field: 'BankName',
                    value5Field: 'BankAccount',
                    value6Field: 'PayeeOperatorName',
                    value7Field: 'ReviewerName',
                    value8Field: 'OperatorName',
                    // window: {
                    //     type: 'App.Control.Kivii.Finances.InvoiceTitle.Selector'
                    // },
                    Type: 'Payee',
                    bind: {
                        key: '{entity.PayeeKvid}',
                        value: '{entity.PayeeName}',
                        value1: '{entity.PayeeTaxNumber}',
                        value2: '{entity.PayeeCompanyAddress}',
                        value3: '{entity.PayeePhone}',
                        value4: '{entity.PayeeRegisteredBank}',
                        value5: '{entity.PayeeBankAccount}',
                        value6: '{entity.Metadata.PayeeOperatorName}',
                        value7: '{entity.Metadata.ReviewerName}',
                        value8: '{entity.OperatorName}'
                    }
                }, {
                    fieldLabel: '纳&nbsp;&nbsp;税&nbsp;号',
                    margin: '0 10 5 10',
                    columnWidth: 1,
                    allowBlank: false,
                    bind: '{entity.PayeeTaxNumber}'
                }, {
                    xtype: 'comboSelector',
                    // columnWidth: 1,
                    // margin: '10 10 5 10',
                    fieldLabel: '地&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;址',
                    dataSource: '/codet/CurrentMember/InvoiceTitle/Address.json',
                    editable: false,
                    readOnly: false,
                    allowBlank: false,
                    keyField: 'Kvid',
                    valueField: 'CompanyAddress',
                    value3Field: 'Phone',
                    value6Field: 'PayeeOperatorName',
                    value7Field: 'ReviewerName',
                    value8Field: 'OperatorName',
                    // window: {
                    //     type: 'App.Control.Kivii.Finances.InvoiceTitle.Selector'
                    // },
                    bind: {
                        value: '{entity.PayeeCompanyAddress}',
                        value3: '{entity.PayeePhone}',
                        value6: '{entity.Metadata.PayeeOperatorName}',
                        value7: '{entity.Metadata.ReviewerName}',
                        value8: '{entity.OperatorName}'
                    }
                    // },{
                    //             fieldLabel: '地&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;址',
                    //             bind: '{entity.PayeeCompanyAddress}'
                }, {
                    fieldLabel: '电&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;话',
                    bind: '{entity.PayeePhone}'
                }, {
                    fieldLabel: '开&nbsp;&nbsp;户&nbsp;行',
                    bind: '{entity.PayeeRegisteredBank}'
                }, {
                    fieldLabel: '账&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;号',
                    bind: '{entity.PayeeBankAccount}'
                }]
            }]
        }]
    }, {
        xtype: 'container',
        flex: 1,
        layout: 'fit',
        margin: '10 10 10 10',
        items: [{
            xtype: 'container',
            columnWidth: 1,
            style: {
                border: '1px solid #C1C1C1'
            },
            height: 190,
            layout: 'fit',
            items: [{
                xtype: 'entityList',
                reference: 'Details',
                height: 400,
                autoScroll: true,
                viewModel: {
                    data: {
                        _selection: null
                    }
                },
                bind: {
                    selection: '{_selection}'
                },
                store: {
                    pageSize: 400,
                    autoSync: false,
                    listeners: {
                        datachanged: function(_ctrl) {
                            let _view = Ext.getCmp('execute');
                            let _vm = _view.getViewModel();
                            let _totalAmountUntaxed = 0; //未税总额
                            let _totalAmountTax = 0; //总税额
                            let _totalAmount = 0; //含税总额
                            for (let _i = 0; _i < _ctrl.data.length; _i++) {
                                _totalAmountUntaxed = parseFloat((_totalAmountUntaxed + parseFloat(_ctrl.data.items[_i].data.AmountUntaxed)).toFixed(2));
                                _totalAmountTax = parseFloat((_totalAmountTax + parseFloat(_ctrl.data.items[_i].data.AmountTax)).toFixed(2));
                                _totalAmount = parseFloat((_totalAmount + parseFloat(_ctrl.data.items[_i].data.Amount)).toFixed(2));
                            }
                            _vm.set('entity.AmountUntaxed', _totalAmountUntaxed);
                            _vm.set('entity.AmountTax', _totalAmountTax);
                            _vm.set('entity.Amount', _totalAmount);
                        }
                    }
                },
                dockedItems: [{
                    xtype: 'toolbar',
                    dock: 'top',
                    items: [{
                        margin: '0 0 -9 3',
                        xtype: 'displayfield',
                        bind: {
                            value: '选中列以编辑'
                        },
                        fieldStyle: {
                            fontSize: '20px'
                        }

                    }, '->', {
                        text: '编辑',
                        iconCls: 'x-fa fa-edit',
                        iconAlign: 'left',
                        handler: '_onEditDetail',
                        bind: {
                            disabled: '{!_selection}'
                        }
                    }, {
                        text: '添加',
                        iconCls: 'x-fa fa-edit',
                        iconAlign: 'left',
                        handler: '_onAddDetail'
                    }]
                }, {
                    xtype: 'toolbar',
                    dock: 'bottom',
                    height: 45,
                    margin: '0 0 -5 0',
                    style: {
                        'border-top': '1px solid red'
                    },
                    items: ['->', {
                        xtype: 'displayfield',
                        value: '总额(不含税):'
                    }, {
                        xtype: 'displayfield',
                        bind: {
                            value: '￥{entity.AmountUntaxed}'
                        }
                    }, '-', {
                        xtype: 'displayfield',
                        value: '总税额:'
                    }, {
                        xtype: 'displayfield',
                        bind: {
                            value: '￥{entity.AmountTax}'
                        }
                    }, '-', {
                        xtype: 'displayfield',
                        value: '总额(含税):'
                    }, {
                        xtype: 'displayfield',
                        bind: {
                            value: '￥{entity.Amount} ({amountChineseBig})'
                        }
                    }]
                }],
                columns: [{
                    xtype: 'rownumberer',
                    align: 'center'
                }, {
                    text: '组别<span style="color:green;font-size:16px;margin-top:10px;">&nbsp;*</span>',
                    dataIndex: 'GoodsId',
                    width: 120
                }, {
                    text: '名称<span style="color:green;font-size:16px;margin-top:10px;">&nbsp;*</span>',
                    dataIndex: 'GoodsFullName',
                    width: 150,
                    editor: {
                        xtype: 'textfield',
                        enforceMaxLength: true,
                        maxLength: 50,
                        allowBlank: false
                    }
                }, {
                    text: '规格型号',
                    dataIndex: 'GoodsSpecifications',
                    flex: 1,
                    editor: {
                        xtype: 'textfield'
                    }
                }, {
                    text: '单位<span style="color:green;font-size:16px;margin-top:10px;">&nbsp;*</span>',
                    dataIndex: 'QuantityUnit',
                    width: 80,
                    editor: {
                        xtype: 'textfield',
                        enforceMaxLength: true,
                        maxLength: 20,
                        allowBlank: true
                    }
                }, {
                    text: '数量',
                    dataIndex: 'Quantity',
                    width: 80,
                    editor: {
                        xtype: 'numberfield',
                        hideTrigger: true,
                        // enforceMaxLength: true,
                        listeners: {
                            change: '_onChangeQuantity'
                        }
                    }
                }, {
                    text: '单价(不含税)',
                    dataIndex: 'QuantityUnitPriceUntaxed',
                    hidden: true,
                    width: 120,
                    align: 'right',
                    renderer: function(_value) {
                        let _newValue = Ext.util.Format.usMoney(_value);
                        let _newValue1 = _newValue.replace('$', '¥');
                        return _newValue1;
                    }
                }, {
                    text: '金额(不含税)',
                    hidden: true,
                    dataIndex: 'AmountUntaxed',
                    width: 180,
                    align: 'right',
                    renderer: function(_value) {
                        let _newValue = Ext.util.Format.usMoney(_value);
                        let _newValue1 = _newValue.replace('$', '¥');
                        return _newValue1;
                    }
                }, {
                    text: '单价(含税)',
                    dataIndex: 'QuantityUnitPrice',
                    width: 150,
                    align: 'right',
                    renderer: function(_value) {
                        let _newValue = Ext.util.Format.usMoney(_value);
                        let _newValue1 = _newValue.replace('$', '¥');
                        return _newValue1;
                    }
                }, {
                    text: '金额(含税)',
                    dataIndex: 'Amount',
                    width: 180,
                    align: 'right',
                    renderer: function(_value) {
                        let _newValue = Ext.util.Format.usMoney(_value);
                        let _newValue1 = _newValue.replace('$', '¥');
                        return _newValue1;
                    },
                    editor: {
                        xtype: 'numberfield',
                        hideTrigger: true,
                        enforceMaxLength: true,
                        listeners: {
                            change: '_onChangeAmount'
                        }
                    }
                }, {
                    text: '税率',
                    dataIndex: 'TaxRate',
                    align: 'right',
                    width: 100,
                    renderer: function(_value, _meta, _record) {
                        if (_value === undefined) return 0;
                        return _value * 100 + '%';
                    }
                }, {
                    text: '税额',
                    width: 120,
                    align: 'right',
                    dataIndex: 'AmountTax',
                    renderer: function(_value) {
                        let _newValue = Ext.util.Format.usMoney(_value);
                        let _newValue1 = _newValue.replace('$', '¥');
                        return _newValue1;
                    }
                }, {
                    xtype: 'actioncolumn',
                    width: 80,
                    text: '操作',
                    align: 'center',
                    menuDisabled: true,
                    sortable: false,
                    items: [{
                        iconCls: 'x-fa fa-trash-o',
                        handler: '_onDeleteDetail'
                    }]
                }],
                controller: {
                    _onChangeQuantity: function(_ctrl, _newValue, _oldValue, _eOpts) {
                        let _controller = this;
                        let _vm = _controller.getViewModel();
                        let _view = _controller.getView();
                        let _value = _newValue;
                        let _detail = _view.selection;
                        _detail.data.Quantity = _value;
                        _detail.data.QuantityUnitPrice = (parseFloat(_detail.data.Amount)/parseFloat(_detail.data.Quantity)).toFixed(2); //parseFloat(_detail.data.Amount.toFixed(2))/parseFloat(_detail.data.Quantity); //单价(含税)
                        _detail.data.QuantityUnitPriceUntaxed = (parseFloat(_detail.data.AmountUntaxed)/parseFloat(_detail.data.Quantity)).toFixed(2); //parseFloat(_detail.data.AmountUntaxed.toFixed(2))/parseFloat(_detail.data.Quantity); //单价(不含税)

                        _view.modifyEntity(_detail.data);
                    },
                    _onChangeAmount: function(_ctrl, _newValue, _oldValue, _eOpts) {
                        let _controller = this;
                        let _vm = _controller.getViewModel();
                        let _view = _controller.getView();
                        let _value = _newValue;
                        let _detail = _view.selection;
                        _detail.data.Amount = (parseFloat(_value)).toFixed(2);

                        _detail.data.AmountUntaxed = (parseFloat(_detail.data.Amount)/(1 + parseFloat(_detail.data.TaxRate))).toFixed(2); //parseFloat(_detail.data.Amount.toFixed(2))/(1+parseFloat(_detail.data.TaxRate.toFixed(2))); //金额(未税)
                        _detail.data.AmountTax = (parseFloat(_detail.data.Amount) - parseFloat(_detail.data.AmountUntaxed)).toFixed(2); //parseFloat(_detail.data.AmountUntaxed.toFixed(2))*_detail.data.TaxRate; //税额
                        _detail.data.QuantityUnitPrice = (parseFloat(_detail.data.Amount)/parseFloat(_detail.data.Quantity)).toFixed(2); //parseFloat(_detail.data.Amount.toFixed(2))/parseFloat(_detail.data.Quantity); //单价(含税)
                        _detail.data.QuantityUnitPriceUntaxed = (parseFloat(_detail.data.AmountUntaxed)/parseFloat(_detail.data.Quantity)).toFixed(2); //parseFloat(_detail.data.AmountUntaxed.toFixed(2))/parseFloat(_detail.data.Quantity); //单价(不含税)

                        _view.modifyEntity(_detail.data);
                    },
                    _onEditDetail: function(_ctrl) {
                        let _controller = this;
                        let _view = _controller.getView();
                        let _vm = _controller.getViewModel();
                        if (!_view.selection) return;
                        let _data = _view.getStore().getData().items;
                        let _form = Ext.create('App.Control.Kivii.Finances.InvoiceApplyDetail.Form', {
                            header: false
                        });
                        let _win = Ext.create('Ext.window.Window', {
                            layout: 'fit',
                            title: _ctrl.text,
                            iconCls: _ctrl.iconCls,
                            // header:false,
                            iconAlign: 'left',
                            autoScroll: true,
                            width: 900,
                            height: 550,
                            modal: true,
                            resizable: false,
                            items: [_form]
                        });
                        _win.show();
                        _form.setOwner(_view.selection, _data);
                        _form.on('entityUpdated', function(_ctrl, _entities) {
                            _win.close();
                            _view.modifyEntity(_entities);
                            _view.clearSelections();
                        });
                    },
                    _onAddDetail: function(_ctrl) {
                        let _controller = this;
                        let _view = _controller.getView();
                        let _store = _view.getStore();
                        let _data = _store.getData().items;
                        let _detail = {};
                        _detail.GoodsId = "发票1";
                        _detail.GoodsFullName = "检测费";
                        _detail.GoodsModel = "3040601000000000000";
                        _detail.TaxRate = 0.03; //_item.TaxRate;
                        _detail.Quantity = 1;
                        _detail.Amount = 0;
                        _detail.QuantityUnit = '次';
                        _detail.AmountUntaxed = (parseFloat(_detail.Amount)/(1 + parseFloat(_detail.TaxRate))).toFixed(2);
                        _detail.AmountTax = (parseFloat(_detail.Amount) - parseFloat(_detail.AmountUntaxed)).toFixed(2);
                        _detail.QuantityUnitPrice = (parseFloat(_detail.Amount)/parseFloat(_detail.Quantity)).toFixed(2);
                        _detail.QuantityUnitPriceUntaxed = (parseFloat(_detail.AmountUntaxed)/parseFloat(_detail.Quantity)).toFixed(2);
                        let _form = Ext.create('App.Control.Kivii.Finances.InvoiceApplyDetail.Form', {
                            header: false
                        });
                        let _win = Ext.create('Ext.window.Window', {
                            layout: 'fit',
                            title: _ctrl.text,
                            iconCls: _ctrl.iconCls,
                            // header:false,
                            iconAlign: 'left',
                            autoScroll: true,
                            width: 900,
                            height: 550,
                            modal: true,
                            resizable: false,
                            items: [_form]
                        });
                        _win.show();
                        _form.setOwner(_detail, _data, true);
                        _form.on('entityAdd', function(_ctrl, _entities) {
                            _win.close();
                            _view.addEntity(_entities);
                            _view.clearSelections();
                            // _view.reload();
                        });
                    },
                    _onDeleteDetail: function(_ctrl, _rowIndex, _colIndex, _item, _e, _record) {
                        let _controller = this;
                        let _vm = _controller.getViewModel();
                        let _kvid = _vm.data.entity.Kvid;
                        if (_kvid !== null) {
                            Ext.toast("已存在发票,无法移除明细!");
                            return;
                        }
                        let _view = _controller.getView();
                        _view.removeEntity(_record.data);
                    }
                }
            }]
        }]
    }],
    setOwner: function(_item, _edit) {
        let _view = this;
        let _vm = _view.getViewModel();
        let _refs = _view.getReferences();
        let _store = _refs.Details.getStore();
        if (!Ext.isObject(_item)) return; //如果不是object类型，直接返回
        let _entity = {};
        Ext.apply(_entity, _item);
        _vm.set('entity', _entity);
        if (_entity.Kvid !== null) {
            _vm.set('isEdit', false);
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Invoice/Read.json',
                params: {
                    IncludeDetail: true,
                    Kvid: _entity.Kvid
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _view.readEntity(_rtns.Result, false);
                    for (let _i = 0; _i < _rtns.Result.Details.length; _i++) {
                        let _record = _rtns.Result.Details[_i];
                        _store.add(_record);
                    }
                },
                failure: function(_response) {
                    Ext.toast('请重试!');
                }
            });
            return;
        }
        if (_edit === null || _edit === undefined) _edit = false;
        _vm.set('isEdit', _edit);
        if (Ext.isArray(_entity.Payments)) _vm.set('payments', _entity.Payments);
        if (Ext.isArray(_entity.Details)) _store.add(_entity.Details);
        if (_entity.PayerName !== null) {
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.InvoiceTitle/Query.json',
                params: {
                    CompanyName: _entity.PayerName
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    if (_rtns.Results.length <= 0) return Ext.toast('未找到对应抬头信息,请先添加!');
                    _vm.set('entity.PayerName', _rtns.Results[0].CompanyName);
                    _vm.set('entity.PayerKvid', _rtns.Results[0].Kvid);
                    _vm.set('entity.PayerTaxNumber', _rtns.Results[0].TaxNumber);
                    _vm.set('entity.PayerCompanyAddress', _rtns.Results[0].CompanyAddress);
                    _vm.set('entity.PayerPhone', _rtns.Results[0].Phone);
                    _vm.set('entity.PayerRegisteredBank', _rtns.Results[0].BankName);
                    _vm.set('entity.PayerBankAccount', _rtns.Results[0].BankAccount);
                },
                failure: function(_response) {
                    Ext.toast('请重试!');
                }
            });
        }
        if (_entity.PayeeName !== null) {
            Ext.Ajax.request({
                url: '/codet/CurrentMember/InvoiceTitle.Json', //'/Restful/Kivii.Finances.Entities.InvoiceTitle/Query.json',
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    if (_rtns.Results.length <= 0) return Ext.toast('未找到对应抬头信息,请先添加!');
                    let _current = _rtns.Results[0];
                    for (let _i = 0; _i < _rtns.Results.length; _i++) {
                        if (_rtns.Results[_i].CompanyName !== _entity.PayeeName) continue;
                        _current = _rtns.Results[_i];
                    }
                    if (_current === null) return;
                    _vm.set('entity.PayeeName', _current.CompanyName);
                    _vm.set('entity.PayeeKvid', _current.Kvid);
                    _vm.set('entity.PayeeTaxNumber', _current.TaxNumber);
                    _vm.set('entity.PayeeCompanyAddress', _current.CompanyAddress);
                    _vm.set('entity.PayeePhone', _current.Phone);
                    _vm.set('entity.PayeeRegisteredBank', _current.BankName);
                    _vm.set('entity.PayeeBankAccount', _current.BankAccount);
                },
                failure: function(_response) {
                    Ext.toast('请重试!');
                }
            });
        }
    },
    controller: {
        _onSaveEntity: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _args = {}; //设置通过参数
            _args.Item={};
            for (let _property in _vm.data.entity) {
                if (_property != 'Metadata' && _vm.data.entity[_property] === _vm.data.remoteEntity[_property]) continue;
                _args.Item[_property] = _vm.data.entity[_property];
            }
            _args.Item.Kvid = _vm.data.entity.Kvid;
            _view._isActivate && _view.mask('数据通讯中...');
            Ext.Ajax.request({
		        url: '/Restful/Kivii.Finances.Entities.Invoice/UpdateEx.json',
		        jsonData: _args,
		        success: function(_response) {
		            var _rtns = Ext.decode(_response.responseText);
		            var _remoteEntity = _vm.data.remoteEntity;
		            Ext.apply(_remoteEntity, _rtns.Results[0]);
		            _vm.set('remoteEntity',_remoteEntity);
		            var _entity={};
		            Ext.apply(_entity,_remoteEntity);
		            _vm.set('entity', _entity);
		            _view.fireEvent('entityUpdated', _view, _entity);
		        },
		        failure: function(_response) {
		            _view._isActivate &&_view.unmask();
		        }
		    });
        },
        _onEditEntity: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _store = _view.down('gridpanel').getStore().data;
            let _item = _vm.data.entity;
            _item.Details = [];
            for (let i = 0; i < _store.items.length; i++) {
                if (_store.items[i].data.Amount <= 0) continue;
                _item.Details.push(_store.items[i].data);
            }
            _view.fireEvent('entityEdit', _view, _item);
        },
        _onAcceptEntity: function() {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _store = _view.down('gridpanel').getStore().data;
            let _item = _vm.data.entity;
            _item.Details = [];
            for (let i = 0; i < _store.items.length; i++) {
                if (_store.items[i].data.Amount <= 0) continue;
                _item.Details.push(_store.items[i].data);
            }
            let _jsonData = {};
            _jsonData.Items = [];
            _jsonData.Items.push(_item);
            if (_vm.data.payments.length > 0) _jsonData.Payments = _vm.data.payments;
            // if (_item.Amount !== _vm.data.owner.Amount) {
            //     Ext.toast("申请明细金额与申请原始金额不一致!");
            //     return;
            // }
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Invoice/Accept.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('新增成功');
                    _view.fireEvent('entityAccepted', _controller, _rtns);
                },
                failure: function(_response) {}
            });
        }
    }
});