﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Invoice.MultiAccept.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Invoice',
    xtype: 'App.Control.Kivii.Finances.Invoice.MultiAccept.List',
    title: '批录入发票',
    iconCls: 'x-fa fa-ticket',
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar: true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer: false,
    viewModel: {
        data: {
            _selection: null,
            amountSelected: 0,
            entity: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: false,
        autoSync: false,
        pageSize: 400,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
            margin: '0 0 -9 3',
            xtype: 'displayfield',
            bind: {
                value: '所选到账总额 {entity.Amount} 元/已添加发票金额:{amountSelected}元'
            },
            fieldStyle: {
                fontSize: '16px'
            }
        }, '->', {
            text: '添加',
            iconCls: 'x-fa fa-plus',
            iconAlign: 'left',
            handler: '_onAddInvoice'
        }, {
            text: '录入',
            iconCls: 'x-fa fa-ticket',
            iconAlign: 'left',
            handler: '_onAcceptInvoice'
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            xtype: 'actioncolumn',
            align: 'center',
            text: '操作',
            items: [{
                //     text: '编辑',
                //     iconCls: 'x-fa fa-pencil-square-o',
                //     handler: '_onEditDetail'
                // }, '->', {
                text: '移除',
                iconCls: 'x-fa fa-trash-o',
                handler: '_onRemoveDetail'
            }],
            width: 80
        }, {
            text: '发票编码',
            dataIndex: 'SerialNumber',
            width: 90
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            width: 100
        }, {
            text: '发票抬头',
            dataIndex: 'PayerName',
            renderer: '_renderPayerName',
            width: 200
        }, {
            text: '金额(含税)',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 110
        }, {
            text: '税额',
            align: 'right',
            dataIndex: 'AmountTax',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 100
        }, {
            text: '金额(不含税)',
            align: 'right',
            dataIndex: 'AmountUntaxed',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 120
        }, {
            xtype: 'datecolumn',
            text: '开票日期',
            format: 'Y-m-d',
            dataIndex: 'OperateTime',
            width: 100
        }, {
            text: '申请人',
            dataIndex: 'OperatorName',
            width: 140
        }, {
            text: '类型',
            dataIndex: 'Type',
            width: 130
        }, {
            text: '分类',
            dataIndex: 'Category',
            renderer: '_renderCategory',
            width: 90
        }, {
            text: '税号',
            dataIndex: 'PayerTaxNumber',
            width: 200
        }, {
            text: '地址',
            dataIndex: 'PayerCompanyAddress',
            renderer: '_renderPayerCompanyAddress',
            width: 260
        }, {
            text: '开户行',
            dataIndex: 'PayerRegisteredBank',
            renderer: '_renderPayerRegisteredBank',
            width: 260
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 180,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    setOwner: function(_item) {
        let _view = this;
        let _vm = _view.getViewModel();
        if (!Ext.isObject(_item)) return; //如果不是object类型，直接返回
        let _entity = {};
        Ext.apply(_entity, _item);
        _vm.set('entity', _entity);
        if (Ext.isArray(_entity.Payments)) _vm.set('payments', _entity.Payments);
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderPayerName: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== '00000000-0000-0000-0000-000000000000') {
                _meta.style = 'background-color:#FFC0CB';
                return _value + '[已作废]';
            }
            return _value;
        },
        _renderCategory: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Debit") {
                _meta.style = 'background-color:#F3F781';
                return "借票";
            }
            if (_value === "Payment") {
                _meta.style = 'background-color:#B4EEB4';
                return "到账";
            }
            return _value;
        },
        _renderPayerCompanyAddress: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerPhone;
        },
        _renderPayerRegisteredBank: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerBankAccount;
        },

        _reCalculateAmount: function() {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _store = _view.getStore();
            let _records = _store.getData().items;
            let _amountSelected = 0.0;
            for (let _i = 0; _i < _records.length; _i++) {
                _amountSelected += parseFloat(_records[_i].data.Amount);
            }
            _vm.set('amountSelected', _amountSelected.toFixed(2));
        },
        _onRemoveDetail: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            _record.drop();
            _controller._reCalculateAmount();
        },
        _onAddInvoice: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _store = _view.getStore();
            let _exists = _store.getData().items;
            let _entity = _vm.data.entity;
            if(!_entity){
            	Ext.toast('无到账信息,无法添加!');
                return;
            }
            let _existAmount = 0;
            for (let _i = 0; _i < _exists.length; _i++) {
                let _record = _exists[_i].data;
                _existAmount += parseFloat(_record.Amount);
            }
            if (_existAmount >= _entity.Amount) {
                Ext.toast('已添加发票金额已满足,无法继续添加!');
                return;
            }
            let _item = {};
            Ext.apply(_item, _entity);
            _item.Amount = _entity.Amount - _existAmount;
            _item.Details = [];
            let _detail = {};
            _detail.GoodsId = "发票1";
            _detail.GoodsFullName = "检测费";
            _detail.GoodsModel = "3040601000000000000";
            _detail.TaxRate = _item.TaxRate;
            _detail.Quantity = 1;
            _detail.Amount = parseFloat(_item.Amount.toFixed(2));
            _detail.QuantityUnit = '次';
            _detail.AmountUntaxed = (parseFloat(_detail.Amount) / (1 + parseFloat(_detail.TaxRate))).toFixed(2);
            _detail.AmountTax = (parseFloat(_detail.Amount) - parseFloat(_detail.AmountUntaxed)).toFixed(2);
            _detail.QuantityUnitPrice = (parseFloat(_detail.Amount) / parseFloat(_detail.Quantity)).toFixed(2);
            _detail.QuantityUnitPriceUntaxed = (parseFloat(_detail.AmountUntaxed) / parseFloat(_detail.Quantity)).toFixed(2);
            _item.Details.push(_detail);
            let _form = Ext.create('App.Control.Kivii.Finances.Invoice.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 750,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_item, true);
            _form.on('entityEdit', function(_ctrl, _editItem) {
                _win.close();
                if (_item.Amount < _editItem.Amount) {
                    Ext.toast("编辑金额超过可开票金额!");
                    return;
                }
                _store.add(_editItem);
                _controller._reCalculateAmount();
            });
        },
        _onAcceptInvoice:function(_ctrl){
        	let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _store = _view.getStore();
            let _exists = _store.getData().items;
            let _jsonData = {};
            _jsonData.Items = [];
            for (let _i = 0; _i < _exists.length; _i++) {
                let _record = _exists[_i].data;
            	_jsonData.Items.push(_record);
            }
            if (_vm.data.payments.length > 0) _jsonData.Payments = _vm.data.payments;
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Invoice/Accept.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('批量新增成功');
                    _view.fireEvent('entityAccepted', _controller, _rtns);
                },
                failure: function(_response) {}
            });
        }
    }
});