﻿//解决rowwidget不能触发collapsebody和expandbody事件
Ext.override(Ext.grid.plugin.RowExpander, {
    relayedEvents: ['collapsebody', 'expandbody'],
    init: function(grid) {
        //--给grid注册expandbody 和collapsebody事件
        var me = this,
            ownerLockable = grid.ownerLockable;
        grid.rowExpanderEventRelayers = grid.relayEvents(grid.view, me.relayedEvents);
        if (ownerLockable) {
            ownerLockable.editorEventRelayers = ownerLockable.relayEvents(ownerLockable.view, me.relayedEvents);
        }
        this.callParent([grid]);

    },
    //重写getHeaderConfig，使下级没有数据时，可以隐藏加号,通过haveRowExpander判断
    getHeaderConfig: function() {
        var me = this,
            lockable = me.grid.lockable && me.grid;

        return {
            width: me.headerWidth,
            ignoreExport: true,
            lockable: false,
            autoLock: true,
            sortable: false,
            resizable: false,
            draggable: false,
            hideable: false,
            menuDisabled: true,
            tdCls: Ext.baseCSSPrefix + 'grid-cell-special',
            innerCls: Ext.baseCSSPrefix + 'grid-cell-inner-row-expander',
            renderer: function(value, meta, record) {
                if (me.haveRowExpander) {
                    if (me.haveRowExpander(record)) {
                        return '<div class="' + Ext.baseCSSPrefix + 'grid-row-expander" role="presentation" tabIndex="0"></div>';
                    } else {
                        return "";
                    }
                } else {
                    return '<div class="' + Ext.baseCSSPrefix + 'grid-row-expander" role="presentation" tabIndex="0"></div>';
                }
            },
            processEvent: function(type, view, cell, rowIndex, cellIndex, e, record) {
                var isTouch = e.pointerType === 'touch',
                    isExpanderClick = !!e.getTarget('.' + Ext.baseCSSPrefix + 'grid-row-expander');

                if ((type === "click" && isExpanderClick) || (type === 'keydown' && e.getKey() === e.SPACE)) {

                    // Focus the cell on real touch tap.
                    // This is because the toggleRow saves and restores focus
                    // which may be elsewhere than clicked on causing a scroll jump.
                    if (isTouch) {
                        cell.focus();
                    }
                    me.toggleRow(rowIndex, record, e);
                    e.stopSelection = !me.selectRowOnExpand;
                } else if (e.type === 'mousedown' && !isTouch && isExpanderClick) {
                    e.preventDefault();
                }
            },

            // This column always migrates to the locked side if the locked side is visible.
            // It has to report this correctly so that editors can position things correctly
            isLocked: function() {
                return lockable && (lockable.lockedGrid.isVisible() || this.locked);
            },

            // In an editor, this shows nothing.
            editRenderer: function() {
                return ' ';
            }
        };
    }

});
Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Payment.Used.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.Control.Kivii.Finances.Payment.Used.List',
    dataSource: '/Restful/Kivii.Finances.Entities.Payment/UsedQuery.json',
    title: '使用记录',
    iconCls: 'x-fa fa-sign-in',
    filter: {
        AmountNotEqualTo: 0
        // OperateTimeGreaterThanOrEqualTo: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        // OperateTimeLessThan: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1)
    },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar: true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer: false,
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'CreateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                xtype: 'datefield',
                fieldLabel: '操作日期 开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanBeginTime'
                    }
                },
                // value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanEndTime'
                    }
                },
                // value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'SerialNumber,PayerName,PayeeName,PayeeAccountName,PayeeAccountOwnerName,PayeeAccountSerialNumber',
                // queryField: 'Amount',
                emptyText: '模糊搜索流水号、付款单位、充值账户名、充值账号或名称',
            }, {
                text: '充值撤销',
                iconCls: 'x-fa fa-sign-out',
                handler: '_onTransferCancel',
                bind: {
                    disabled: '{!_selection}'
                }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '流水号',
            dataIndex: 'SerialNumber',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            width: 220
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: '_renderAmount',
            width: 110
        }, {
            text: '类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        }, {
            xtype: 'datecolumn',
            text: '操作日期',
            format: 'Y-m-d H:i',
            dataIndex: 'CreateTime',
            width: 150
        }, {
            text: '操作人',
            dataIndex: 'OperatorName',
            width: 100
        }, {
            text: '充值账户',
            dataIndex: 'PayeeAccountName',
            width: 280
        }, {
            text: '账户所属',
            dataIndex: 'PayeeAccountOwnerName',
            width: 150
        }, {
            text: '充值账号',
            dataIndex: 'PayeeAccountSerialNumber',
            width: 180
        }, {
            text: '备注',
            dataIndex: 'Remark',
            flex: 1
        }]
    },
    plugins: [{
        ptype: 'gridfilters'
    }, {
        ptype: 'rowwidget',
        widget: {
            xtype: 'grid',
            reference: 'settlement',
            autoLoad: true,
            store: {
                pageSize: 10000,
                data: [],
                autoLoad: false
            },
            border: '1px #d0d0d0 solid',
            margin: '0 0 0 20',
            features: [{
                ftype: 'summary'
            }],
            columns: {
                defaults: {
                    menuDisabled: true,
                    sortable: false
                },
                items: [{
                    xtype: 'rownumberer'
                }, {
                    dataIndex: 'BizId',
                    text: '所属业务',
                    width: 170,
                    // sortable: true,
                    summaryType: 'sum',
                    summaryRenderer: function(value, summaryData, dataIndex) {
                        return Ext.String.format('合计');
                    }
                }, {
                    dataIndex: 'OwnerName',
                    text: '所属部门',
                    width: 120
                }, {
                    dataIndex: 'OperateTime',
                    text: '受理日期',
                    width: 100,
                    // sortable: true,
                    renderer: Ext.util.Format.dateRenderer('Y-m-d')
                }, {
                    dataIndex: 'AmountPlan',
                    align: 'right',
                    text: '原金额',
                    width: 100,
                    renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
                    summaryType: 'sum',
                    summaryRenderer: function(value, summaryData, dataIndex) {
                        return Ext.String.format('￥{0}', value.toFixed(2));
                    }
                }, {
                    dataIndex: 'Amount',
                    align: 'right',
                    text: '结算金额',
                    width: 100,
                    renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
                    summaryType: 'sum',
                    summaryRenderer: function(value, summaryData, dataIndex) {
                        return Ext.String.format('￥{0}', value.toFixed(2));
                    }
                }, {
                    dataIndex: 'AmountPayment',
                    align: 'right',
                    text: '已付金额',
                    width: 100,
                    renderer: '_renderAmountDetail',
                    summaryType: 'sum',
                    summaryRenderer: function(value, summaryData, dataIndex) {
                        return Ext.String.format('￥{0}', value.toFixed(2));
                    }
                }, {
                    dataIndex: 'CreateTime',
                    text: '结算日期',
                    width: 100,
                    // sortable: true,
                    renderer: Ext.util.Format.dateRenderer('Y-m-d')
                }, {
                    dataIndex: 'PayedTime',
                    text: '付款日期',
                    width: 100,
                    // sortable: true,
                    renderer: Ext.util.Format.dateRenderer('Y-m-d')
                }, {
                    dataIndex: 'Metadata.TOTALPRICE',
                    align: 'right',
                    text: '报告金额',
                    width: 100,
                    renderer: '_renderObject'
                }, {
                    dataIndex: 'Type',
                    text: '类型',
                    width: 90,
                    renderer: '_renderTypeDetail'
                }, {
                    dataIndex: 'OperatorName',
                    text: '操作人',
                    width: 90
                }, {
                    xtype: 'actioncolumn',
                    text: '操作',
                    align: 'center',
                    items: [{
                        text: '查看',
                        iconCls: 'x-fa fa-search',
                        handler: '_onReadSettlement'
                    }],
                    width: 50
                }, {
                    dataIndex: 'Summary',
                    text: '摘要',
                    width: 200
                }, {
                    dataIndex: 'Remark',
                    text: '备注',
                    minWidth: 100,
                    flex: 1
                }]
            }
        },
        //配置该方法,返回true时,可以展开,否则不能展开
        haveRowExpander:function(_record){
        	if(_record.data.Type==="DynamicPay")return true;
        	return false;
        }
    }],
    listeners: {
        activate: '_onActivate',
        expandbody: '_onExpandBody'
    },
    setOwner: function(_owner, _ctrl) {
        let _view = this;
        if (_view.filter === null) _view.setFilter({
            AmountNotEqualTo: 0
        });
        _view.filter.ParentKvid = _owner.Kvid;
        _view.reload();
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderObject: function(_value, _meta, _record, _rowIndex, _cellIndex, _store, _grid) { //对Metadata的Render的默认方法
            var _dataIndex = _meta.column.dataIndex;
            if (!_dataIndex) return _value;
            var _pos = _dataIndex.indexOf('.');
            if (_pos < 1) return _value;
            var _objName = _dataIndex.substr(0, _pos);
            if (!_record.data[_objName]) return '';
            if (!Ext.isObject(_record.data[_objName])) return '';
            var _propertyName = _dataIndex.substr(_pos + 1);
            return _record.data[_objName][_propertyName];
        },
        _renderTypeDetail: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Contract") {
                return "合同费";
            }
            if (_value === "Report") {
                return "报告费";
            }
            if (_value === "Service") {
                return "服务费";
            }
            if (_value === "Express") {
                return "快递费";
            }
            return _value;
        },
        _renderAmountDetail: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            } else _meta.style = 'background-color:#F0E68C';
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== null) _meta.style = 'background-color:#FF0000';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== null) _meta.style = 'background-color:#FF0000';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === 'Transfer') {
                return '充值';
            }
            if (_value === 'DynamicPay') {
                return '收款登记';
            }
            return _value;
        },

        //点击+号,展示详情
        _onExpandBody: function(_rowNode, _record, _expandRow, _eOpts) {
            let _controller = this;
            let _refs = _controller.getReferences();
            let _view = _controller.getView();
            let _storeSettlement = _refs.settlement.getStore();
            _storeSettlement.removeAll();
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Settlement/QueryBiz.json',
                jsonData: {
                    PaymentKvid: _record.data.Kvid
                },
                success: function(_response) {
                    if (_response.responseText && _response.responseText.length > 0) {
                        let _rtns = Ext.decode(_response.responseText);
                        for (let _i = 0; _i < _rtns.Results.length; _i++) {
                            _storeSettlement.add(_rtns.Results[_i]);
                        }
                    }
                },
                failure: function(_response, _opts) {}
            });
        },
        _onReadSettlement: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _view = _controller.getView();
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 680,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_record.data);
            _form.on('entityDeleted', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
            _form.on('entityUpdated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.CreateTimeGreaterThanOrEqualTo;
            delete _view.filter.CreateTimeLessThan;
            _view.filter.CreateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.CreateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _cleanBeginTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            _view.reload();
        },
        _cleanEndTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OperateTimeLessThan;
            _view.reload();
        },
        _onTransferCancel: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            let _msg = Ext.create('Ext.window.MessageBox', {
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                closeAction: 'destroy'
            });
            _msg.confirm(_ctrl.text, '您确认要撤销选中的' + _selecteds.length + '条记录？', _controller._offsetPayments, _controller);

            return;
        },
        _offsetPayments: function(_optional) {
            if (_optional !== 'yes') return;
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            let _kvids = [];
            for (let _i = 0; _i < _selecteds.length; _i++) {
                if (_selecteds[_i].data.Type === "DynamicPay") continue;
                if (_selecteds[_i].data.hasOwnProperty('OffsetKvid') && _selecteds[_i].data.OffsetKvid !== null) continue;
                _kvids.push(_selecteds[_i].data.Kvid);
            }
            if (_kvids.length <= 0) {
                Ext.toast("无符合撤销的数据,请重新选择!");
                return;
            }
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Payment/TransferCancel.json',
                jsonData: {
                    PaymentKvids: _kvids
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('撤销成功！');
                    _view.reload();
                },
                failure: function(_response) {
                    _me.unmask();
                }
            });
        }
    }
});