﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Statistic.Settlement', {
    extend: 'System.Control.EntityList',
    xtype: 'App.View.Kivii.Finances.Statistic.Settlement',
    entityType: 'Kivii.Finances.Entities.Settlement',
    requires: ['System.Control.ComboSelector', 'App.Control.DownloadField'],
    dataSource: '/Restful/Kivii.Finances.Entities.Settlement/Statistic.json',
    globalFilter: {},
    filter: {
        BeginTime: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
        EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
        // TimeType: 'OperateTime',
        Payed: true
    },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar: true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer: false,
    header: true,
    layout: 'fit',
    title: '结算统计',
    iconCls: 'x-fa fa-calculator',
    viewModel: {
        data: {
            // AmountTask: 0,
            // AmountTaskTotal: 0,
            // AmountTaskDiscount: 0,
            // AmountTaskDiscountTotal: 0,
            // AmountPackage: 0,
            // AmountCalculation: 0,
            // AmountReport: 0,
            // AmountReportChange: 0,
            // AmountPayment: 0,
            // DiscountRate: 0,
            // reportQuantity: 0,
            // xReportQuantity: 0,
            _selection: null,
            lastOwnerKvid: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'CreateTime',
            direction: 'ASC'
        }]
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
            xtype: 'comboSelector',
            dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
            reference: 'depart',
            responsiveCls: 'big-100 small-100',
            keyField: 'Kvid',
            valueField: 'FullName',
            key: KiviiContext.CurrentMember.DepartmentKvid,
            value: KiviiContext.CurrentMember.DepartmentName,
            editable: false,
            width: 300,
            allowBlank: true,
            // triggers: {
            //     clear: {
            //         weight: 2,
            //         cls: 'x-fa fa-times',
            //         handler: '_cleanSelect'
            //     }
            // },
            listeners: {
                select: '_onSelect'
            }
        }, {
            xtype: 'combo',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'payed',
            value: true,
            width: 100,
            editable: false,
            readOnly: false,
            filterPickList: true,
            reference: 'payed',
            store: {
                fields: ['name', 'payed'],
                data: [{
                    name: '已收费',
                    payed: true
                }, {
                    name: '未收费',
                    payed: false
                }, {
                    name: '全部',
                    payed: null
                }]
            },
            listeners: {
                select: '_onSelectPayed'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '时间范围  开始',
            format: 'Y-m-d',
            labelWidth: 90,
            width: 220,
            reference: 'startTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '结束',
            format: 'Y-m-d',
            labelWidth: 30,
            width: 160,
            reference: 'endTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth(), 0),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'searchField',
            flex: 1,
            labelWidth: 60,
            queryField: 'PayerName,BizId',
            emptyText: '模糊搜索付款单位、所属报告'
        }, {
            xtype: 'downloadField',
            dataSource: '/Restful/Kivii.Finances.Entities.Settlement/Statistic.xlsx', //数据源
            fileName: '数据导出',
            window: {
                type: 'App.Control.Kivii.Finances.General.Template.Selector',
                title: '选择模板',
                iconCls: 'x-fa fa-file-excel-o',
                autoScroll: true,
                width: 700,
                height: 350,
                closeAction: 'hide',
                modal: true,
                layout: 'fit',
                filter: {
                    FolderPath: '/Finances/Settlement/Files',
                    TagsContains: 'Statistic'
                }
            }
        }]
    }, {
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            margin: '0 0 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '报告费: {AmountSettlement}&nbsp;&nbsp;已收费: {AmountPayment}&nbsp;&nbsp;未收费: {AmountUnpaid}&nbsp;&nbsp;已作废: {AmountOffset}&nbsp;&nbsp;批次:{QuantityPayment}/{QuantitySettlement}(作废:{QuantityOffset})'
            },
            fieldStyle: {
                fontSize: '14px'
            }
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer'
        }, {
            dataIndex: 'BizId',
            text: '所属报告',
            width: 170,
            sortable: true,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('合计');
            }
        }, {
            dataIndex: 'PayerName',
            text: '付款单位',
            width: 150,
            sortable: true
        }, {
            dataIndex: 'OperateTime',
            text: '委托日期',
            width: 100,
            sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'CreateTime',
            text: '结算日期',
            width: 100,
            sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'Amount',
            align: 'right',
            text: '合计',
            width: 90,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'AmountPayment',
            align: 'right',
            text: '已收款',
            width: 90,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'Type',
            text: '类型',
            width: 90,
            renderer: '_renderType'
        }, {
            dataIndex: 'Category',
            text: '分类',
            width: 90
        }, {
            dataIndex: 'Summary',
            text: '所属附件',
            width: 150
        }, {
            dataIndex: 'OperatorName',
            text: '结算人',
            width: 70
        }, {
            dataIndex: 'Remark',
            text: '备注',
            minWidth: 100,
            flex: 1
        }, {
            dataIndex: 'Metadata.TOTALPRICE',
            text: 'Lims原价',
            width: 120
        }, {
            dataIndex: 'Metadata.COMPNAME',
            text: '委托单位',
            width: 120
        }, {
            dataIndex: 'Metadata.NAMEOFPRODUCTIONORG',
            text: '生产单位',
            width: 120
        }, {
            dataIndex: 'Metadata.RELEASEDDATE',
            text: '签发日期',
            width: 100
        }, {
            dataIndex: 'Metadata.PayedName',
            text: '付款信息',
            width: 120
        }, {
            dataIndex: 'Metadata.PayedSerialNumber',
            text: '付款流水',
            width: 120
        }]
    },
    listeners: {
        activate: '_onActivate',
        viewready: '_viewReady'
    },
    controller: {
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Contract") {
                return "合同费";
            }
            if (_value === "Report") {
                return "报告费";
            }
            if (_value === "Service") {
                return "服务费";
            }
            if (_value === "Express") {
                return "快递费";
            }
            return _value;
        },
        _viewReady: function() {
            var _me = this;
            var _view = _me.getView();
            var _vm = _me.getViewModel();
            var _store = _view.getStore();
            _store.on('load', function(_this, _records, _successful, _operation, _eOpts) {
                let _rtns = Ext.decode(_operation.getResponse().responseText);
                _vm.set('AmountSettlement', _rtns.TotalAmount.toFixed(2));
                _vm.set('AmountPayment', _rtns.TotalAmountPayment.toFixed(2));
                _vm.set('AmountUnpaid', _rtns.TotalAmountUnpaid.toFixed(2));
                _vm.set('AmountOffset', _rtns.TotalAmountOffset.toFixed(2));
                _vm.set('Quantity', _rtns.TotalQuantity.toFixed(0));
                _vm.set('QuantityPayment', _rtns.TotalQuantityPayment.toFixed(0));
                _vm.set('QuantityUnpaid', _rtns.TotalQuantityUnpaid.toFixed(0));
                _vm.set('QuantitySettlement', _rtns.TotalQuantity.toFixed(0));
                _vm.set('QuantityOffset', _rtns.TotalQuantityOffset.toFixed(0));
            });
        },
        _onSelectPayed: function(_ctrl, _record) {
            var _controller = this,
                _view = _controller.getView(),
                _vm = _controller.getViewModel();
            delete _view.filter.Payed;
            _view.filter.Payed = _record.data.payed;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            var _controller = this;
            var _view = _controller.getView();
            var _refs = _controller.getReferences();
            var _startTime = _refs.startTime.value;
            var _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.BeginTime;
            delete _view.filter.EndTime;
            _view.filter.BeginTime = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.EndTime = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _onSelect: function(_ctrl, _record, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _ownerKvid = _record.data.Kvid;
            delete _view.filter.OwnerKvid;
            _view.filter.OwnerKvid = _ownerKvid;
            _view.reload();
        },
        _cleanSelect: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OwnerKvid;
            _view.reload();
        }
    }
});