﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "发票明细")]
    [Alias(Configs.TableNameInvoiceDetail)]
    public class InvoiceDetail : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasRemark,
        IEntityHasCreator, IEntityHasUpdater
    {
        #region 关联信息,禁止更新
        [ApiMember(Description = "发票Kvid")]
        [IgnoreUpdate]
        [Required]
        public Guid InvoiceKvid { get; set; }

        #endregion

        #region 发票明细
        
        [ApiMember(Description = "货品编号")]
        //[IgnoreUpdate]
        [StringLength(100), Default("")]
        public string GoodsId { get; set; }

        [ApiMember(Description = "条目名称，货物或应税劳务、服务名称")]
        //[IgnoreUpdate,]
        [StringLength(200), Required]
        public string GoodsFullName { get; set; }

        [ApiMember(Description = "货品规格")]
        //[IgnoreUpdate,]
        [StringLength(200), Default("")]
        public string GoodsSpecifications { get; set; }

        [ApiMember(Description = "货品型号")]
        //[IgnoreUpdate,]
        [StringLength(200), Default("")]
        public string GoodsModel { get; set; }

        [ApiMember(Description = "货品单位")]
        //[IgnoreUpdate,]
        [StringLength(50), Default("")]
        public string GoodsUnit { get; set; }



        [ApiMember(Description = "数量单位")]
        [StringLength(50), Default("")]
        public string QuantityUnit { get; set; }

        [ApiMember(Description = "数量")]
        [DecimalLength(10, 2), Default(0)]
        public decimal Quantity { get; set; }

        [ApiMember(Description = "单价(含税)")]
        [DecimalLength(16, 8), Default(0)]
        public decimal QuantityUnitPrice { get; set; }

        [ApiMember(Description = "单价(不含税)")]
        [DecimalLength(15, 2), Default(0)]
        public decimal QuantityUnitPriceUntaxed { get; set; }

        [ApiMember(Description = "税率")]
        [InternalSetter]
        [DecimalLength(5, 2), Required]
        public decimal TaxRate { get; set; }

        [ApiMember(Description = "含税金额，合同或订单中使用的金额")]
        [DecimalLength(15, 2), Required]
        public decimal Amount { get; set; }

        [ApiMember(Description = "未税金额，Amount=AmountUntaxed+AmountTax")]
        [DecimalLength(15, 2),Default(0)]
        public decimal AmountUntaxed { get; set; }

        [ApiMember(Description = "税额,和计算结果AmountUntaxed*TaxRate不能相差超过一个固定值")]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountTax { get; set; }
        #endregion

        #region 备注
        [ApiMember(Description = "备注")]
        [StringLength(1000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作人Creator Updater
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion

    }
}
