﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "优惠政策")]
    [Alias(Configs.TableNamePolicy)]
    public class Policy : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasSummary,
        IEntityHasRemark, IEntityHasCreator, IEntityHasUpdater
    {
        [ApiMember(Description = "所属Kvid")]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [ApiMember(Description = "所属名称")]
        [StringLength(200), Default("")]
        public string OwnerName { get; set; }

        /// <summary>
        /// 用于描述政策的名称
        /// </summary>
        [ApiMember(Description = "名称")]
        [StringLength(200), Default("")]
        public string Name { get; set; }

        /// <summary>
        /// 用于区别当前是原金额基础之上的政策或是折上折政策或其他情况
        /// </summary>
        [ApiMember(Description = "类型")]
        [StringLength(50), Required]
        public PolicyType Type { get; set; }

        /// <summary>
        /// 政策减免金额,如果设置了金额就不取折扣
        /// </summary>
        [DecimalLength(10, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "货币单位")]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }

        /// <summary>
        /// 折扣。折扣不能大于1能等于0
        /// </summary>
        [ApiMember(Description = "折扣率")]
        [DecimalLength(4, 2), Default(1)]
        public decimal? DiscountRate { get; set; }

        [ApiMember(Description = "生效日期")]
        [Required]
        public DateTime EffectiveTime { get; set; }

        [ApiMember(Description = "失效日期")]
        public DateTime? ExpiredTime { get; set; }

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        //[IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime 

        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经办人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion
    }

    [Api(Description = "优惠政策明细")]
    [Alias(Configs.TableNamePolicyDetail)]
    public class PolicyDetail : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasCreator, IEntityHasUpdater
    {
        [IgnoreUpdate]
        [Required]
        public Guid PolicyKvid { get; set; }

        [StringLength(200), Default("")]
        public string BizId { get; set; }

        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [StringLength(200), Default("")]
        public string BizType { get; set; }

        [ApiMember(Description = "目标单位")]
        [StringLength(200), Default("")]
        public string TargetName { get; set; }

        [DefaultEmptyGuid]
        public Guid TargetKvid { get; set; }

        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
    }

    [Api(Description = "政策执行记录")]
    [Alias(Configs.TableNamePolicyRecord)]
    public class PolicyRecord : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasOffset,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        [IgnoreUpdate]
        [Required]
        public Guid PolicyKvid { get; set; }

        [IgnoreUpdate]
        [Required]
        public Guid SettlementKvid { get; set; }

        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }

        [StringLength(200), Default("")]
        public string BizId { get; set; }

        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [StringLength(200), Default("")]
        public string BizType { get; set; }

        [ApiMember(Description = "目标单位")]
        [StringLength(200), Default("")]
        public string TargetName { get; set; }

        [DefaultEmptyGuid]
        public Guid TargetKvid { get; set; }

        [ApiMember(Description = "原金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPlan { get; set; }

        [ApiMember(Description = "原折后金额")]//AmountOrigin=AmountPolicy+Amount
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountOrigin { get; set; }

        [ApiMember(Description = "减免金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPolicy { get; set; }

        [ApiMember(Description = "合计金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        #region 操作相关  Organization Creator Updater Operator OperateTime 

        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经 办 人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大3000字")]
        [StringLength(3000), Default("")]
        public string Remark { get; set; }
        #endregion

    }

    public enum PolicyType
    {
        /// <summary>
        /// 普通折扣
        /// </summary>
        Original,

        /// <summary>
        /// 附加折扣,折上折
        /// </summary>
        Additional
    }
}
