﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "借票冲销")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceDebiting : RestfulExecution<Invoice>
    {
        //public Guid ApplyKvid { get; set; }
        public Guid InvoiceKvid { get; set; }
        public List<Guid> InvoiceKvids { get; set; }
        public Guid PaymentKvid { get; set; }
        //public AccountType Type { get; set; }
        //public decimal Amount { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (PaymentKvid == Guid.Empty) throw new Exception("请传入到账信息!");
            if (InvoiceKvid == Guid.Empty && InvoiceKvids.IsNullOrEmpty()) throw new Exception("请传入要冲销的发票!");
            if (InvoiceKvids.IsNullOrEmpty()) InvoiceKvids = new List<Guid>();
            if (InvoiceKvid != Guid.Empty) InvoiceKvids.Add(InvoiceKvid);
            //if (Type != AccountType.Bank && Type != AccountType.Cash && Type != AccountType.Pos) throw new Exception("冲销类型不正确!");
            //if (Type != AccountType.Bank && Amount <= 0) throw new Exception("冲销金额不能小于0!");
            //if (Type == AccountType.Bank && PaymentKvid == Guid.Empty) throw new Exception("请选择到账进行冲销!");

            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var invoices = conn.SelectByIds<Invoice>(InvoiceKvids);
            if (invoices.IsNullOrEmpty()) throw new Exception("未找到所选发票信息!");
            if (invoices.Exists(o => o.OffsetKvid != Guid.Empty || o.RootKvid != o.Kvid || o.Amount <= 0)) throw new Exception("存在不正确的发票类型");
            if (invoices.Count != InvoiceKvids.Count) throw new Exception("所选发票信息与查询后结果不一致!");
            var unDebitAmount = invoices.Sum(o => o.Amount) - invoices.Sum(o => o.AmountPayment);
            (unDebitAmount <= 0).ThrowIfTrue("此发票已冲销,请勿重复操作!");
            //var applyKvids = invoices.ConvertAll(o => o.ApplyKvid).Distinct().ToList();
            //var applys = conn.SelectByIds<InvoiceApply>(applyKvids);
            ////applys.ThrowIfNullOrEmpty("未找到所选借票申请数据!");
            ////(applys.OperateType != InvoiceApplyType.Debit).ThrowIfTrue("非借票不可冲销!");
            ////(applys.OffsetKvid != Guid.Empty).ThrowIfTrue("当前借票申请已冲账,无法继续操作!");
            ////(applys.Status != (int)InvoiceApplyStatus.Completed).ThrowIfTrue("当前借票申请未执行完毕,无法冲销!");
            //var applyRelations = new List<InvoiceApply>();
            //if (!applys.IsNullOrEmpty()) applyRelations = conn.Select<InvoiceApply>(o => o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty && Sql.In(o.ParentKvid, applyKvids));

            var rtns = new RestfulUpdateResponse<Invoice>();
            rtns.Results = new List<Invoice>();

            var payment = conn.SingleById<Payment>(PaymentKvid);
            if (payment == null) throw new Exception("未找到收款信息!");
            if (payment.Type != PaymentType.Bank && payment.Type != PaymentType.Split && payment.Type != PaymentType.Assign && payment.Type != PaymentType.Cash && payment.Type != PaymentType.Pos && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat) throw new Exception("请选择正确银行到账进行冲销!");
            if (payment.AmountInvoice >= payment.Amount) throw new Exception("当前到账已开票,无法冲销当前借票!");
            var splitPayments = conn.Select<Payment>(o => o.ParentKvid == payment.Kvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.UnBiz, PaymentType.Refund, PaymentType.Split, PaymentType.Assign));
            var parentPayment = conn.Single<Payment>(o => o.OffsetKvid == Guid.Empty && o.Kvid == payment.ParentKvid);
            if (!splitPayments.IsNullOrEmpty())
            {
                var queryApplying = conn.From<InvoiceApply>();
                //只查询存在 申请中的 apply,查看当前正在申请中的金额  用作判断是否允许被冲销
                queryApplying.Where(o => o.AmountInvoice == 0 && o.OperateType == InvoiceApplyType.Related && o.OffsetKvid == Guid.Empty && Sql.In(o.BizKvid, splitPayments.ConvertAll(p => p.Kvid)));
                var existApplyings = conn.Select(queryApplying);
                (!existApplyings.IsNullOrEmpty()).ThrowIfTrue("所选到账,存在开票中的申请!请开票完成后再操作!");
            }
            //if (Amount <= 0 || Amount > (payment.Amount - payment.AmountInvoice)) Amount = payment.Amount - payment.AmountInvoice;
            //decimal amount = Amount >= unDebitAmount ? unDebitAmount : Amount;

            var payments = new List<Payment>();
            payments.Add(payment);
            var correlatedInvoices = invoices.Correlating(payments);//生成payment和invoice之间的关联,同事原Invoices也要更新相关数据

            if (!splitPayments.IsNullOrEmpty())//要是所选Payment存在拆分项,满足条件就要更新AmountInvoice
            {
                //自定义排序,把属于当前部门的认领到账优先处理
                splitPayments = splitPayments.OrderBy(o =>
                {
                    var index = 0;
                    if (o.OwnerKvid == KiviiContext.CurrentMember.OrganizationKvid) index = 0;
                    else if (o.OwnerKvid == KiviiContext.CurrentMember.DepartmentKvid) index = 1;
                    else if (o.OwnerKvid == KiviiContext.CurrentMember.Kvid) index = 2;
                    else index = int.MaxValue;
                    return index;
                }).ToList();
                decimal amountInvoice = payment.AmountInvoice;
                foreach (var item in splitPayments)
                {
                    if (amountInvoice <= 0) continue;
                    item.AmountInvoice = item.Amount >= amountInvoice ? amountInvoice : item.Amount;
                    item.AddOnlyProperties(o => o.AmountInvoice);
                    amountInvoice -= item.AmountInvoice;
                }
            }

            var trans = conn.OpenTransaction();
            try
            {
                //conn.Insert(relation);
                //conn.UpdateOnly(applys);
                conn.UpdateOnly(payment);
                if (!splitPayments.IsNullOrEmpty())
                {
                    foreach (var item in splitPayments)
                    {
                        if (item.OnlyProperties.IsNullOrEmpty()) continue;
                        conn.UpdateOnly(item);
                    }
                }
                if (parentPayment != null)
                {
                    parentPayment.RecalculateAmountInvoice(conn);
                }
                foreach (var item in invoices)
                {
                    if (item.OnlyProperties.Count > 0)
                    {
                        conn.UpdateOnly(item);
                        item.RemoveAllOnlyProperties();
                        rtns.Results.Add(item);
                    }
                }
                foreach (var item in correlatedInvoices)
                {
                    item.SerialNumber = item.GetSubSerialNumber(conn);
                    conn.Insert(item);
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [Api(Description = "冲销撤销")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceDebitCancel : RestfulExecution<Invoice>
    {
        public Guid InvoiceKvid { get; set; }
        public List<Guid> InvoiceKvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (InvoiceKvid == Guid.Empty && InvoiceKvids.IsNullOrEmpty()) throw new Exception("请传入要撤销冲销的发票!");
            if (InvoiceKvids.IsNullOrEmpty()) InvoiceKvids = new List<Guid>();
            if (InvoiceKvid != Guid.Empty) InvoiceKvids.Add(InvoiceKvid);
            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var invoices = conn.SelectByIds<Invoice>(InvoiceKvids);
            if (invoices.IsNullOrEmpty()) throw new Exception("未找到所选发票信息!");
            if (invoices.Exists(o => o.OffsetKvid != Guid.Empty || o.RootKvid != o.Kvid || o.Amount <= 0)) throw new Exception("存在不正确的发票类型");
            if (invoices.Count != InvoiceKvids.Count) throw new Exception("所选发票信息与查询后结果不一致!");
            var relationInvoices = conn.Select<Invoice>(o => Sql.In(o.ParentKvid, InvoiceKvids) && o.OffsetKvid == Guid.Empty && o.Type == "Relation");
            if (relationInvoices.IsNullOrEmpty()) throw new Exception("所选发票未找到冲销记录,无需冲销!");
            var existPayments = conn.Select<Payment>(o => Sql.In(o.Kvid, relationInvoices.ConvertAll(p => p.BizKvid)));//Type:Bank,Pos,Cash,WeChat...
            if (existPayments.IsNullOrEmpty()) throw new Exception("所选发票未找到冲销记录,无需撤销!");
            var splitPayments = conn.Select<Payment>(o => o.OffsetKvid == Guid.Empty && o.Type == PaymentType.Split && Sql.In(o.ParentKvid, existPayments.ConvertAll(p => p.Kvid)));

            foreach (var item in existPayments)
            {
                var relations = relationInvoices.Where(o => o.BizKvid == item.Kvid).ToList();
                if (!relations.IsNullOrEmpty())
                {
                    item.AmountInvoice -= relations.Sum(o => o.Amount);
                    if (item.AmountInvoice < 0) item.AmountInvoice = 0;
                    item.AddOnlyProperties(o => o.AmountInvoice);
                }
                if (!splitPayments.IsNullOrEmpty())//要是所选Payment存在拆分项,满足条件就要更新AmountInvoice
                {
                    decimal amountInvoice = 0;
                    amountInvoice = item.AmountInvoice;
                    foreach (var split in splitPayments)
                    {
                        if (amountInvoice <= 0) split.AmountInvoice = 0;
                        else split.AmountInvoice = split.Amount >= amountInvoice ? amountInvoice : split.Amount;
                        split.AddOnlyProperties(o => o.AmountInvoice);
                        amountInvoice -= split.AmountInvoice;
                    }
                }
            }
            var rtns = new RestfulUpdateResponse<Invoice>();
            rtns.Results = new List<Invoice>();

            var trans = conn.OpenTransaction();
            try
            {
                //删除关联Payment的发票数据
                var updateRelationInvoices = conn.From<Invoice>();
                updateRelationInvoices = updateRelationInvoices.Update(o => o.Status);
                updateRelationInvoices = updateRelationInvoices.Where(o => Sql.In(o.Kvid, relationInvoices.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<Invoice>(new Invoice { Status = -1 }, updateRelationInvoices);
                //更新发票的付款时间和到账金额
                var updateExistInvoices = conn.From<Invoice>();
                updateExistInvoices = updateExistInvoices.Update(o => new { o.PayedTime, o.AmountPayment });
                updateExistInvoices = updateExistInvoices.Where(o => Sql.In(o.Kvid, invoices.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<Invoice>(new Invoice { PayedTime = null, AmountPayment = 0 }, updateExistInvoices);

                //更新已关联的payment的AmountInvoice
                foreach (var item in existPayments)
                {
                    if (item.OnlyProperties.Count > 0) conn.UpdateOnly(item);
                }
                //更新已关联的payment的父级或子级AmountInvoice
                if (!splitPayments.IsNullOrEmpty())
                {
                    foreach (var item in splitPayments)
                    {
                        if (item.OnlyProperties.Count <= 0) continue;
                        conn.UpdateOnly(item);
                        item.RecalculateAssignAmount(conn);
                    }
                }
                rtns.Results.AddRange(invoices);

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
